@@ -64,14 +64,20 @@ if __name__ == '__main__':     #enclosing required because of multiprocessing
     try:
         from lauetoolsnn.utils_lauenn import generate_classHKL, generate_multimat_dataset, \
                                         rmv_freq_class_MM, get_multimaterial_detail,\
-                                        array_generator, array_generator_verify, vali_array, LoggingCallback
+                                        array_generator, array_generator_verify, vali_array
+        from lauetoolsnn.NNmodels import model_arch_general_optimized, LoggingCallback,\
+                                    model_arch_general_onelayer, model_arch_CNN_DNN_optimized
     except:
         # else import from a path where LaueToolsNN files are
         import sys
         sys.path.append(r"C:\Users\purushot\Desktop\github_version_simple\lauetoolsnn")
         from utils_lauenn import generate_classHKL, generate_multimat_dataset, \
                                 rmv_freq_class_MM, get_multimaterial_detail,\
-                                array_generator, array_generator_verify, vali_array, LoggingCallback
+                                array_generator, array_generator_verify, vali_array
+        from NNmodels import model_arch_general_optimized, LoggingCallback,\
+                                    model_arch_general_onelayer, model_arch_CNN_DNN_optimized
+                                
+                                
     
     # ## step 1: define material and other parameters for simulating Laue patterns
         
@@ -280,173 +286,7 @@ if __name__ == '__main__':     #enclosing required because of multiprocessing
         n_bins = len(angbins)-1
         n_outputs = len(classhkl)
         print(n_bins, n_outputs)
-        # ## Step 3: Defining a neural network architecture
         
-        # In[4]:
-        import tensorflow as tf
-        import keras
-        from keras.regularizers import l2
-        from keras.models import Sequential
-        from keras.layers import Dense, Activation, Dropout
-        from tensorflow.keras import Model
-
-        metricsNN = [
-                    keras.metrics.FalseNegatives(name="fn"),
-                    keras.metrics.FalsePositives(name="fp"),
-                    keras.metrics.TrueNegatives(name="tn"),
-                    keras.metrics.TruePositives(name="tp"),
-                    keras.metrics.Precision(name="precision"),
-                    keras.metrics.Recall(name="accuracy"),
-                    ]
-        def model_arch_CNN_DNN_optimized(shape, 
-                                         layer_activation="relu", 
-                                         output_activation="softmax",
-                                         dropout=0.3,
-                                         stride = [1,1],
-                                         kernel_size = [5,5],
-                                         pool_size=[2,2],
-                                         CNN_layers = 2,
-                                         CNN_filters = [32,64],
-                                         DNN_layers = 3,
-                                         DNN_filters = [1000,500,100],
-                                         output_neurons = 11,
-                                         learning_rate = 0.001,
-                                         output="DNN"):            
-            inputs = keras.layers.Input(shape, name="InputLayer")
-            
-            for lay in range(CNN_layers):
-                if lay == 0:
-                    conv1 = keras.layers.Conv1D(filters=CNN_filters[lay], kernel_size=kernel_size[lay], 
-                                                strides=stride[lay], 
-                                                activation=layer_activation, name="Conv_"+str(lay+1))(inputs)
-                    pool1 = keras.layers.MaxPooling1D(pool_size=pool_size[lay], \
-                                                      name="Pool_"+str(lay+1))(conv1)
-                else:
-                    conv1 = keras.layers.Conv1D(filters=CNN_filters[lay], kernel_size=kernel_size[lay], 
-                                                strides=stride[lay], 
-                                                activation=layer_activation, name="Conv_"+str(lay+1))(pool1)
-                    pool1 = keras.layers.MaxPooling1D(pool_size=pool_size[lay], \
-                                                      name="Pool_"+str(lay+1))(conv1)
-            flatten = keras.layers.Flatten(name="Flatten")(pool1)
-
-            for lay in range(DNN_layers):
-                if lay == 0:
-                    ppKL = keras.layers.Dense(DNN_filters[lay], activation=layer_activation, \
-                                              name="Dense_"+str(lay+1))(flatten)
-                    ppKL = keras.layers.Dropout(dropout, name="Dropout"+str(lay+1))(ppKL)   
-                else:
-                    
-                    ppKL = keras.layers.Dense(DNN_filters[lay], activation=layer_activation, \
-                                              name="Dense_"+str(lay+1))(ppKL)
-                    ppKL = keras.layers.Dropout(dropout, name="Dropout"+str(lay+1))(ppKL) 
-            ## Output layer 
-            if output != "CNN":
-                if DNN_layers == 0:
-                    outputs = keras.layers.Dense(output_neurons, activation=output_activation, name="Dense_out")(flatten)
-                else:
-                    outputs = keras.layers.Dense(output_neurons, activation=output_activation, name="Dense_out")(ppKL)
-            else:
-                outputs = keras.layers.Conv1D(filters=output_neurons, kernel_size=1, 
-                                            strides=1, activation=output_activation, name="Conv_out")(flatten)
-            model = Model(inputs, outputs)
-            ## Compile model
-            otp = tf.keras.optimizers.Adam(learning_rate=learning_rate)
-            model.compile(loss='categorical_crossentropy', optimizer=otp, metrics=[metricsNN])
-            model.summary()
-            return model
-        
-        def model_arch_general_optimized(n_bins, n_outputs, kernel_coeff = 0.0005, bias_coeff = 0.0005, lr=None, verbose=1,
-                                write_to_console=None):
-            """
-            Very simple and straight forward Neural Network with few hyperparameters
-            straighforward RELU activation strategy with cross entropy to identify the HKL
-            Tried BatchNormalization --> no significant impact
-            Tried weighted approach --> not better for HCP
-            Trying Regularaization 
-            l2(0.001) means that every coefficient in the weight matrix of the layer 
-            will add 0.001 * weight_coefficient_value**2 to the total loss of the network
-            1e-3,1e-5,1e-6
-            """
-            if n_outputs >= n_bins:
-                param = n_bins
-                if param*15 < (2*n_outputs): ## quick hack; make Proper implementation
-                    param = (n_bins + n_outputs)//2
-            else:
-                param = n_outputs*2 ## More reasonable ???
-                
-            model = Sequential()
-            model.add(keras.Input(shape=(n_bins,)))
-            ## Hidden layer 1
-            model.add(Dense(n_bins, kernel_regularizer=l2(kernel_coeff), bias_regularizer=l2(bias_coeff)))
-            # model.add(BatchNormalization())
-            model.add(Activation('relu'))
-            model.add(Dropout(0.3)) ## Adding dropout as we introduce some uncertain data with noise
-            ## Hidden layer 2
-            model.add(Dense(((param)*15 + n_bins)//2, kernel_regularizer=l2(kernel_coeff), bias_regularizer=l2(bias_coeff)))
-            # model.add(BatchNormalization())
-            model.add(Activation('relu'))
-            model.add(Dropout(0.3))
-            ## Hidden layer 3
-            model.add(Dense((param)*15, kernel_regularizer=l2(kernel_coeff), bias_regularizer=l2(bias_coeff)))
-            # model.add(BatchNormalization())
-            model.add(Activation('relu'))
-            model.add(Dropout(0.3))
-            ## Output layer 
-            model.add(Dense(n_outputs, activation='softmax'))
-            ## Compile model
-            if lr != None:
-                otp = tf.keras.optimizers.Adam(learning_rate=lr)
-                model.compile(loss='categorical_crossentropy', optimizer=otp, metrics=[metricsNN])
-            else:
-                model.compile(loss='categorical_crossentropy', optimizer="adam", metrics=[metricsNN])
-            
-            if verbose == 1:
-                model.summary()
-                stringlist = []
-                model.summary(print_fn=lambda x: stringlist.append(x))
-                short_model_summary = "\n".join(stringlist)
-                if write_to_console!=None:
-                    write_to_console(short_model_summary)
-            return model
-        
-        def model_arch_general_onelayer(n_bins, n_outputs, 
-                                        kernel_coeff = 0.0005, 
-                                        bias_coeff = 0.0005, lr=None, verbose=1,
-                                        write_to_console=None):
-            """
-            Very simple and straight forward Neural Network with few hyperparameters
-            straighforward RELU activation strategy with cross entropy to identify the HKL
-            Tried BatchNormalization --> no significant impact
-            Tried weighted approach --> not better for HCP
-            Trying Regularaization 
-            l2(0.001) means that every coefficient in the weight matrix of the layer 
-            will add 0.001 * weight_coefficient_value**2 to the total loss of the network
-            1e-3,1e-5,1e-6
-            """
-            model = Sequential()
-            model.add(keras.Input(shape=(n_bins,)))
-            ## Hidden layer 1
-            model.add(Dense(n_bins, kernel_regularizer=l2(kernel_coeff), bias_regularizer=l2(bias_coeff)))
-            # model.add(BatchNormalization())
-            model.add(Activation('relu'))
-            model.add(Dropout(0.3)) ## Adding dropout as we introduce some uncertain data with noise
-            ## Output layer 
-            model.add(Dense(n_outputs, activation='softmax'))
-            ## Compile model
-            if lr != None:
-                otp = tf.keras.optimizers.Adam(learning_rate=lr)
-                model.compile(loss='categorical_crossentropy', optimizer=otp, metrics=[metricsNN])
-            else:
-                model.compile(loss='categorical_crossentropy', optimizer="adam", metrics=[metricsNN])
-            
-            if verbose == 1:
-                model.summary()
-                stringlist = []
-                model.summary(print_fn=lambda x: stringlist.append(x))
-                short_model_summary = "\n".join(stringlist)
-                if write_to_console!=None:
-                    write_to_console(short_model_summary)
-            return model
         # ## Step 4: Training  
             
         # In[5]:
